# -*- coding: utf-8 -*-

# Copyright (c) 2006 - 2007 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the VCS status monitor thread base class.
"""

import os

from PyQt4.QtCore import QThread, QTimer, QString, QStringList, SIGNAL

class VcsStatusMonitorThread(QThread):
    """
    Class implementing the VCS status monitor thread base class.
    
    @signal vcsStatusMonitorData(QStringList) emitted to update the VCS status
    @signal vcsStatusMonitorStatus(QString, QString) emitted to signal the status of the
        monitoring thread (ok, nok, op) and a status message
    """
    def __init__(self, interval, projectDir, parent = None):
        """
        Constructor
        
        @param interval new interval in seconds (integer)
        @param projectDir project directory to monitor (string or QString)
        @param parent reference to the parent object (QObject)
        """
        QThread.__init__(self, parent)
        self.setObjectName("VcsStatusMonitorThread")
        
        self.projectDir = QString(projectDir)
        
        self.interval = interval
        
        self.statusList = QStringList()
        self.reportedStates = {}
    
    def run(self):
        """
        Protected method implementing the tasks action.
        """
        self.timer = QTimer()
        self.timer.setInterval(0)
        self.timer.setSingleShot(True)
        self.connect(self.timer, SIGNAL("timeout()"), self.__timerFired)
        self.timer.start()
        self.exec_()
    
    def setInterval(self, interval):
        """
        Public method to change the monitor interval.
        
        @param interval new interval in seconds (integer)
        """
        self.interval = interval
    
    def stop(self):
        """
        Public method to stop the monitor thread.
        """
        self.exit()

    def __timerFired(self):
        """
        Private slot to handle the timer action.
        """
        self.statusList.clear()
        self.emit(SIGNAL("vcsStatusMonitorStatus(QString, QString)"), 
                  QString("op"), self.trUtf8("Checking repository status"))
        res, statusMsg = self._performMonitor()
        if res:
            status = QString("ok")
        else:
            status = QString("nok")
        self.emit(SIGNAL("vcsStatusMonitorStatus(QString, QString)"), status, statusMsg)
        self.emit(SIGNAL("vcsStatusMonitorData(QStringList)"), self.statusList)
        self.timer.setInterval(self.interval * 1000)
        self.timer.start()

    def _performMonitor(self):
        """
        Protected method implementing the real monitoring action.
        
        This method must be overridden and populate the statusList member variable
        with a list of strings giving the status in the first column and the
        path relative to the project directory starting with the third column.
        The allowed status flags are:
        <li>
            <ul>"A" path was added but not yet comitted</ul>
            <ul>"M" path has local changes</ul>
            <ul>"U" path needs an update</ul>
            <ul>"Z" path contains a conflict</ul>
            <ul>" " path is back at normal</ul>
        </li>
        
        @return tuple of flag indicating successful operation (boolean) and 
            a status message in case of non successful operation (QString)
        """
        raise RuntimeError, 'Not implemented'

    def clearCachedState(self, name):
        """
        Public method to clear the cached VCS state of a file/directory.
        
        @param name name of the entry to be cleared (QString or string)
        """
        key = unicode(name).replace(unicode(self.projectDir)+os.sep, '')
        try:
            del self.reportedStates[key]
        except KeyError:
            pass
